#include "SceneGraph.h"
#include "Polygon3.h"
#include "Mesh.h"
#include "ObjExporter.h"


using namespace std;

namespace GtpVisibilityPreprocessor {

ObjExporter::ObjExporter(const string filename):Exporter(filename)
{
  stream.open(mFilename.c_str());
  stream<<"#File exported by GTP visibility preprocessor"<<endl;
  vertexIndex = 1;
}


ObjExporter::~ObjExporter()
{
  stream<<"# export finished"<<endl;
  stream.close();
}

void
ObjExporter::ExportSceneNode(SceneGraphNode *node)
{
  SceneGraphNodeContainer::iterator ni = node->mChildren.begin();
  for (; ni != node->mChildren.end(); ni++)
    ExportSceneNode(*ni);
  
  
  ObjectContainer::const_iterator mi = node->mGeometry.begin();
  for (; mi != node->mGeometry.end(); mi++) {
    // export the transform...
    ExportIntersectable(*mi);
  }
  
}


void
ObjExporter::ExportMesh(Mesh *mesh)
{
  int vertexOffset = vertexIndex;
  
  VertexContainer::const_iterator vi = mesh->mVertices.begin();
  for (; vi != mesh->mVertices.end(); vi++) {
    stream<<"v "<<(*vi).x<<" "<<(*vi).y<<" "<<(*vi).z<<endl;
	vertexIndex ++;
  }
  
  FaceContainer::const_iterator fi = mesh->mFaces.begin();
  for (; fi != mesh->mFaces.end(); fi++) {
    Face *face = *fi;
    VertexIndexContainer::const_iterator vi = face->mVertexIndices.begin();
	stream<<"f ";
    for (; vi != face->mVertexIndices.end(); vi++)
      stream<<vertexOffset + *vi<<" ";
	stream<<endl;
  }
}


void
ObjExporter::ExportPolygon(Polygon3 *poly)
{
  int vertexOffset = vertexIndex;
  
  VertexContainer::const_iterator vi = poly->mVertices.begin();
  for (; vi != poly->mVertices.end(); vi++) {
    stream<<"v "<<(*vi).x<<" "<<(*vi).y<<" "<<(*vi).z<<endl;
	vertexIndex ++;
  }
  
  stream<<"f ";
  int i = vertexOffset;
  vi = poly->mVertices.begin();
  for (; vi != poly->mVertices.end(); vi++, i++) 
	stream<<i<<" ";
  stream<<endl;
  
}

}
